    

#include <unistd.h>

#include "globdef.h"
#include "uidef.h"
#include "fft1def.h"
#include "fft2def.h"
#include "fft3def.h"
#include "sigdef.h"
#include "screendef.h"
#include "seldef.h"
#include "thrdef.h"
#include "txdef.h"

#define TXBAR_SIZE 256
int tx_pixels_per_bin;
short int *txtrace;
float txad_hz_per_bin;

void tx_bar(int xt,int yt,int val1, int val2)
{
int x, y;
x=xt*text_width;
y=yt*text_height+1;
if(val1 == val2)
  {
  if(val1 >= 0)return;
  lir_fillbox(x,y,TXBAR_SIZE,text_height-2,7);
  return;
  }
if(val2 > val1)
  { 
  lir_fillbox(x+val1,y,val2-val1,text_height-2,12);
  }
else
  {  
  lir_fillbox(x+val2,y,val1-val2,text_height-2,7);
  } 
}

void show_txfft(float *z)
{
int i, ia, ib, k;
float t1;
ia=0;
ib=tx_pixels_per_bin;
for(i=1; i<tx_fftsize; i++)
  {
  lir_line(ia,txtrace[i-1],ib,txtrace[i],0);
  ia=ib;
  ib+=tx_pixels_per_bin;
  }
for(i=0; i<tx_fftsize; i++)
  {
  t1=z[2*i]*z[2*i]+z[2*i+1]*z[2*i+1];
  k=400-0.000001*t1;
  if(k<0)k=0;
  txtrace[i]=k+screen_height-401;
  }    
ia=0;
ib=tx_pixels_per_bin;
for(i=1; i<tx_fftsize; i++)
  {
  lir_line(ia,txtrace[i-1],ib,txtrace[i],15);
  ia=ib;
  ib+=tx_pixels_per_bin;
  }
}

#define TX_SETUP_AD 0
#define BAR_AD_LINE 8


void spproc_setup(void)
{
char s[80];
int i;
int setup_mode;
int ad_pix1;
int ad_pix2;
init_txmem();
if(kill_all_flag)return;
open_tx_input();
if(kill_all_flag)return;
tx_pixels_per_bin=(screen_width-1)/tx_fftsize;
if(tx_pixels_per_bin == 0)
  {
  lirerr(778543);
  return;
  }
txtrace=malloc(tx_fftsize*sizeof(short int));
if(txtrace == NULL)
  {
  lirerr(778543);
  return;
  }
for(i=0; i<tx_fftsize; i++)txtrace[i]=screen_height-1;
lir_sleep(50000);

linrad_thread_create(THREAD_TX_INPUT);
setup_mode=TX_SETUP_AD;
restart:;
clear_screen();
tx_ad_maxamp=0;
ad_pix1=0;
ad_pix2=0;
lir_text(0,BAR_AD_LINE,"Soundcard:");
tx_bar(12,BAR_AD_LINE,-1,-1);
switch (setup_mode)
  {
  case TX_SETUP_AD:
  lir_text(0,0,
            "Verify AF clip level, (some device drivers give too few bits)");
  lir_text(0,1,"Use mixer program to set volume for the AF to never reach");
  lir_text(0,2,"its maximum during normal operation.");
  lir_text(0,3,"Press L or H to change cut-off frequencies");
  sprintf(s,"Low %.0f Hz     High %.0f Hz",tx.lowest_bin*txad_hz_per_bin,
                                       tx.highest_bin*txad_hz_per_bin);
  lir_text(0,4,s);
  
  break;
  
  }
  
while(!kill_all_flag)
  {
memcheck(2,txmem,(int*)&txmem_handle);
// Show the microphone input level. 
  ad_pix2=tx_ad_maxamp;
  if(tx.input_bytes != 2)
    {
    ad_pix2/=0x10000;
    }
  ad_pix2=(ad_pix2*TXBAR_SIZE)/0x8000;    
  tx_bar(12,BAR_AD_LINE,ad_pix1,ad_pix2);
  ad_pix1=ad_pix2;
// 

  tx_ad_maxamp=0.95*tx_ad_maxamp;
  lir_sem_wait(SEM_TXINPUT);
  show_txfft(&tx_fft1[tx_fft1_px]);
  tx_fft1_px=(tx_fft1_px+tx_rf_block)&tx_rfbuf_mask;
  test_keyboard();
  if(lir_inkey != 0)
    {
    goto restart;
    }
  }
lir_join(THREAD_USER_COMMAND);
close_tx_input();
}



void tx_input(void)
{
int ia, ib, pa, pb, pc;
int k, m;
int twosiz;
int nread;
float *z;
twosiz = 2*tx_fftsize;
thread_status_flag[THREAD_TX_INPUT]=THRFLAG_ACTIVE;
while(!kill_all_flag && thread_command_flag[THREAD_TX_INPUT] == THRFLAG_ACTIVE)
  {
  if(tx.input_bytes == 2)
    {
    nread=read(tx_audio_in,&tx_micbuf_shi[mic_pa],tx_read_bytes); 
    }
  else
    {
    nread=read(tx_audio_in,&tx_micbuf_int[mic_pa],tx_read_bytes); 
    }
  if(nread != tx_read_bytes)
    {
    lirerr(1283);
    goto tx_input_x;
    }  
  mic_pa=(mic_pa+tx_input_block)&tx_micbuf_mask;
  if( ((mic_pa-mic_px+tx_micbuf_size)&tx_micbuf_mask) >= 2*tx_fftsize)
    {
// Copy the input to tx_fft1 while multiplying with the window function.
    ib=2*tx_fftsize-1;
    pa=mic_px;
    pb=(pa+ib)&tx_micbuf_mask;
    if(tx.input_bytes == 2)
      {
      if(tx.input_channels==1)
        {
        for( ia=0; ia<tx_fftsize; ia++)
          {
          tx_tmp[tx_fft_permute[ia]]=tx_micbuf_shi[pa]*tx_fft_win[ia];
          if(abs(tx_micbuf_shi[pa]) > tx_ad_maxamp)
            {
            tx_ad_maxamp=abs(tx_micbuf_shi[pa]);
            }
          tx_tmp[tx_fft_permute[ib]]=tx_micbuf_shi[pb]*tx_fft_win[ia];
          ib--; 
          pa=(pa+1)&tx_micbuf_mask;
          pb=(pb+tx_micbuf_mask)&tx_micbuf_mask;
          }
        }
      else
        {
// In case there are two channels for the microphone, just use one of them!!
        for( ia=0; ia<tx_fftsize; ia++)
          {
          tx_tmp[tx_fft_permute[ia]]=tx_micbuf_shi[2*pa]*tx_fft_win[ia];
          if(abs(tx_micbuf_shi[2*pa]) > tx_ad_maxamp)
            {
            tx_ad_maxamp=abs(tx_micbuf_shi[2*pa]);
            }
          tx_tmp[tx_fft_permute[ib]]=tx_micbuf_shi[2*pb]*tx_fft_win[ia];
          ib--; 
          pa=(pa+1)&tx_micbuf_mask;
          pb=(pb+tx_micbuf_mask)&tx_micbuf_mask;
          }
        }  
      }
    else
      {
      if(tx.input_channels==1)
        {
        for( ia=0; ia<tx_fftsize; ia++)
          {
          tx_tmp[tx_fft_permute[ia]]=tx_micbuf_int[pa]*tx_fft_win[ia];
          if(abs(tx_micbuf_int[pa]) > tx_ad_maxamp)
            {
            tx_ad_maxamp=abs(tx_micbuf_int[pa]);
            }
          tx_tmp[tx_fft_permute[ib]]=tx_micbuf_int[pb]*tx_fft_win[ia];
          ib--; 
          pa=(pa+1)&tx_micbuf_mask;
          pb=(pb+tx_micbuf_mask)&tx_micbuf_mask;
          }
        }
      else
        {
// In case there are two channels for the microphone, just use one of them!!
        for( ia=0; ia<tx_fftsize; ia++)
          {
          tx_tmp[tx_fft_permute[ia]]=tx_micbuf_int[2*pa]*tx_fft_win[ia];
          if(abs(tx_micbuf_int[2*pa]) > tx_ad_maxamp)
            {
            tx_ad_maxamp=abs(tx_micbuf_int[2*pa]);
            }
          tx_tmp[tx_fft_permute[ib]]=tx_micbuf_int[2*pb]*tx_fft_win[ia];
          ib--; 
          pa=(pa+1)&tx_micbuf_mask;
          pb=(pb+tx_micbuf_mask)&tx_micbuf_mask;
          }
        }  
      }
    mic_px=(mic_px+tx_input_block)&tx_micbuf_mask;
    fft_real_to_hermitian( tx_tmp, 2*tx_fftsize, tx_fftn, tx_fft_table);
// Output is {Re(z^[0]),...,Re(z^[n/2),Im(z^[n/2-1]),...,Im(z^[1]).
// Clear those parts of the spectrum that we do not want and
// transfer the rest of the points to tx_fft1  
    k=tx.lowest_bin;
    m=tx.highest_bin;
    if(m >= tx_fftsize-1)m=tx_fftsize-2;
    if(k==0)k=1;
    pc=0;
    z=&tx_fft1[tx_fft1_pa];
    while(pc < 2*k)
      {
      z[pc  ]=0;
      z[pc+1]=0;
      pc+=2;
      }
    for(ia=k; ia <= m; ia++)
      {
      z[pc+1]=tx_tmp[ia  ];
      z[pc  ]=tx_tmp[twosiz-ia];
      pc+=2;
      }
    while(pc < twosiz)
      {
      z[pc  ]=0;
      z[pc+1]=0;
      pc+=2;
      }
    tx_fft1_pa=(tx_fft1_pa+tx_rf_block)&tx_rfbuf_mask;
    lir_sem_post(SEM_TXINPUT);
    }
  }
tx_input_x:;
thread_status_flag[THREAD_TX_INPUT]=THRFLAG_RETURNED;
lir_sem_post(SEM_TXINPUT);
while(thread_command_flag[THREAD_TX_INPUT] != THRFLAG_NOT_ACTIVE)
  {
  lir_sleep(1000);
  }
}

void tx_output(void)
{
thread_status_flag[THREAD_TX_OUTPUT]=THRFLAG_ACTIVE;
while(thread_command_flag[THREAD_TX_OUTPUT] == THRFLAG_ACTIVE)
  {
  }
thread_status_flag[THREAD_TX_OUTPUT]=THRFLAG_RETURNED;
while(thread_command_flag[THREAD_TX_OUTPUT] != THRFLAG_NOT_ACTIVE)
  {
  lir_sleep(1000);
  }
}

void init_txmem(void)
{
int i;
// Speech processing is done in both the time domain and in the
// frequency domain.
// The lowest frequency of interest is 150 Hz so it will be
// sufficient to use FFTs that span 10 milliseconds.
// With sin squared windows we use 50% overlap and make a read
// every 5 millisecond. 
// There is a factor of two because the soundcard samples a real-
// valued signal but tx_fftsize is for complex transforms.
tx_fftsize=tx.input_speed*.01;
make_power_of_two(&tx_fftsize);
tx_mask=tx_fftsize-1;
tx_input_block=tx.input_channels*tx_fftsize/2;
tx_read_bytes=tx.input_bytes*tx_input_block;
// Allow the input buffer to hold 160 milliseconds of data (or more)
tx_micbuf_size=32*tx_input_block;
tx_micbuf_mask=tx_micbuf_size-1;
init_memalloc(txmem, MAX_TXMEM_ARRAYS);
if(tx.input_bytes == 2)
  {
  mem(1,&tx_micbuf_shi,tx_micbuf_size*sizeof(short int),0);
  }
else
  {
  mem(1,&tx_micbuf_int,tx_micbuf_size*sizeof(int),0);
  }
tx_rfbuf_size=8*tx_fftsize;
tx_rfbuf_mask=tx_rfbuf_size-1;
mem(12,&tx_fft1,tx_rfbuf_size*sizeof(float),0);
mem(2,&tx_rf1,tx_rfbuf_size*sizeof(float),0);
mem(12,&tx_fft2,tx_rfbuf_size*sizeof(float),0);
mem(3,&tx_rf2,tx_rfbuf_size*sizeof(float),0);
// Allow 0.32 seconds in the output buffer (or more).
tx_output_size=tx.output_speed*0.32;
make_power_of_two(&tx_output_size);
tx_output_mask=tx_output_size-1;
if(tx.output_bytes ==2)
  {
  mem(4,&tx_outbuf_shi,tx_output_size*sizeof(short int),0);
  }
else
  {
  mem(4,&tx_outbuf_int,tx_output_size*sizeof(int),0);
  }
mem(5,&tx_tmp,2*tx_fftsize*sizeof(float),0);
mem(6,&tx_fft_table,tx_fftsize*sizeof(COSIN_TABLE),0);
mem(7,&tx_fft_permute,2*tx_fftsize*sizeof(short int),0);
mem(8,&tx_fft_win,tx_fftsize*sizeof(float),0);
txmem_size=memalloc((int*)(&txmem_handle),"txmem");
if(txmem_size == 0)
  {
  lirerr(1261);
  return;
  }
tx_fftn=0;
i=tx_fftsize;
while(i>1)
  {
  i>>=1;
  tx_fftn++;
  }
make_permute(2, tx_fftn, tx_fftsize, tx_fft_permute);
make_window(2,tx_fftsize, 2, tx_fft_win);
if(tx.input_bytes != 2)
  {
  for(i=0; i<tx_fftsize; i++)tx_fft_win[i]/=0x10000;
  }
for(i=0; i<tx_fftsize; i++)tx_fft_win[i]/=tx_fftsize;

make_sincos(2, tx_fftsize, tx_fft_table); 
mic_pa=0;
mic_px=0;
tx_fft1_pa=0;
tx_fft1_px=0;
tx_rf_block=2*tx_fftsize;
txad_hz_per_bin=(0.5*tx.input_speed)/tx_fftsize;
memcheck(1,txmem,(int*)&txmem_handle);

}
